from __future__ import absolute_import, unicode_literals

import json

from django.contrib.auth.decorators import permission_required
from django.shortcuts import get_object_or_404, render
from django.utils.six import text_type
from django.utils.translation import ugettext as _
from wagtail.wagtailadmin.forms import SearchForm as AdminSearchForm
from wagtail.wagtailadmin.modal_workflow import render_modal_workflow
from wagtail.wagtailsearch.backends import get_search_backend

from ..forms import SearchForm
from ..models import Poll
from ..pagination import paginate


@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def index(request):
    poll_list = Poll.objects.all()
    search_form = SearchForm()

    paginator, page = paginate(
        request,
        Poll.objects.all(),
        per_page=8)

    return render(request, 'wagtailpolls/index.html', {
        'page': page,
        'paginator': paginator,
        'poll_list': poll_list,
        'search_form': search_form,
    })


@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def search(request):
    poll_list = Poll.objects.all()
    search_form = SearchForm(request.GET or None)

    if search_form.is_valid():
        query = search_form.cleaned_data['query']
        poll_list = poll_list.search(query)

    else:
        paginator, page = paginate(
            request,
            Poll.objects.all(),
            per_page=8)

    paginator, page = paginate(
        request,
        poll_list,
        per_page=20)

    return render(request, 'wagtailpolls/search.html', {
        'page': page,
        'paginator': paginator,
        'poll_list': poll_list,
        'search_form': search_form,
    })


def choose(request):
    items = Poll.objects.all()

    # Search
    is_searching = False
    search_query = None
    if 'q' in request.GET:
        search_form = AdminSearchForm(request.GET, placeholder=_("Search %(snippet_type_name)s") % {
            'snippet_type_name': 'Polls'
        })

        if search_form.is_valid():
            search_query = search_form.cleaned_data['q']

            search_backend = get_search_backend()
            items = search_backend.search(search_query, items)
            is_searching = True

    else:
        search_form = AdminSearchForm()

    # Pagination
    paginator, paginated_items = paginate(request, items, per_page=25)

    # If paginating or searching, render "results.html"
    if request.GET.get('results', None) == 'true':
        return render(request, "wagtailpolls/search_results.html", {
            'items': paginated_items,
            'query_string': search_query,
            'is_searching': is_searching,
        })

    return render_modal_workflow(
        request,
        'wagtailpolls/choose.html', 'wagtailpolls/choose.js',
        {
            'snippet_type_name': 'Poll',
            'items': paginated_items,
            'is_searchable': True,
            'search_form': search_form,
            'query_string': search_query,
            'is_searching': is_searching,
        }
    )


def chosen(request, id):
    model = Poll
    item = get_object_or_404(model, id=id)

    snippet_json = json.dumps({
        'id': item.id,
        'string': text_type(item),
    })

    return render_modal_workflow(
        request,
        None, 'wagtailpolls/chosen.js',
        {
            'snippet_json': snippet_json,
        }
    )



from django.contrib import messages
from django.contrib.auth.decorators import permission_required
from django.shortcuts import redirect, render, get_object_or_404
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from django.utils.lru_cache import lru_cache

from wagtail.wagtailadmin.edit_handlers import (
    ObjectList, extract_panel_definitions_from_model_class)
from wagtail.wagtailcore.models import Page


@lru_cache(maxsize=None)
def get_poll_edit_handler(Poll):
    panels = extract_panel_definitions_from_model_class(Poll)
    EditHandler = ObjectList(panels).bind_to_model(Poll)
    return EditHandler


@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def create(request):
    from ..models import Poll

    poll = Poll()
    EditHandler = get_poll_edit_handler(Poll)
    EditForm = EditHandler.get_form_class(Poll)

    if request.method == 'POST':
        form = EditForm(request.POST, request.FILES, instance=poll)
        if form.is_valid():
            poll = form.save()
            poll.save()
            messages.success(request, _('The poll "{0!s}" has been added').format(poll))
            return redirect('wagtailpolls_index')

        else:
            messages.error(request, _('The poll could not be created due to validation errors'))
            edit_handler = EditHandler(instance=poll, form=form)

    else:
        form = EditForm(instance=poll)
        edit_handler = EditHandler(instance=poll, form=form)

    return render(request, 'wagtailpolls/create.html', {
        'form': form,
        'edit_handler': edit_handler,
    })


@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def edit(request, poll_pk):
    from ..models import Poll

    poll = get_object_or_404(Poll, pk=poll_pk)

    EditHandler = get_poll_edit_handler(Poll)
    EditForm = EditHandler.get_form_class(Poll)

    if request.method == 'POST':
        form = EditForm(request.POST, request.FILES, instance=poll)

        if form.is_valid():
            poll = form.save()
            poll.save()
            messages.success(request, _('The poll "{0!s}" has been updated').format(poll))
            return redirect('wagtailpolls_index')

        else:
            messages.error(request, _('The poll could not be updated due to validation errors'))
            edit_handler = EditHandler(instance=poll, form=form)
    else:
        form = EditForm(instance=poll)
        edit_handler = EditHandler(instance=poll, form=form)

    return render(request, 'wagtailpolls/edit.html', {
        'poll': poll,
        'form': form,
        'edit_handler': edit_handler,
    })


@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def delete(request, poll_pk):
    from ..models import Poll

    poll = get_object_or_404(Poll, pk=poll_pk)

    if request.method == 'POST':
        poll.delete()
        return redirect('wagtailpolls_index')

    return render(request, 'wagtailpolls/delete.html', {
        'poll': poll,
    })

@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def copy(request, poll_pk):
    from ..models import Poll

    poll = Poll.objects.get(id=poll_pk)

    if request.method == 'POST':
        poll.pk = None
        poll.save()
        return redirect('wagtailpolls_index')

    return render(request, 'wagtailpolls/copy.html', {
        'poll': poll,
    })



from __future__ import unicode_literals, absolute_import

from django.conf.urls import include, url
from django.core import urlresolvers
from django.utils.translation import ugettext_lazy as _

from wagtail.wagtailcore import hooks
from . import urls
from wagtail.wagtailadmin.menu import MenuItem


@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^polls/', include(urls)),
    ]


@hooks.register('construct_main_menu')
def construct_main_menu(request, menu_items):
    menu_items.append(
        MenuItem(_('Polls'), urlresolvers.reverse('wagtailpolls_index'),
                 classnames='icon icon-group', order=250)
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-25 13:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailpolls', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='poll',
            name='title',
            field=models.CharField(max_length=128, verbose_name='Titre'),
        ),
        migrations.AlterField(
            model_name='question',
            name='question',
            field=models.CharField(max_length=128, verbose_name='Question'),
        ),
    ]



from django.conf import settings
from django.contrib.auth.decorators import permission_required
from django.http import Http404, HttpResponse, JsonResponse
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from wagtailpolls.forms import VoteForm
from wagtailpolls.models import Poll, Vote


def vote_data(poll):
    questions = poll.questions.all()
    votes = Vote.objects.filter(question__poll=poll)
    _vote_data = {
        'poll': poll.title,
        'total_questions': questions.count(),
        'total_votes': votes.count(),
        'votes': {
            question.question: question.votes.count()
            for question in questions
            }
    }
    return _vote_data


def _vote(request, poll_pk):
    """
    Performs the vote

    :param request: http request
    :param poll_pk: poll identifier
    :returns: JSON response
    """
    try:
        int(poll_pk)
    except ValueError:
        raise Http404("<h1>{0}</h1>".format('Oops, there is no poll to vote on!'))

    poll = get_object_or_404(Poll, pk=poll_pk)

    form = VoteForm(data=request.POST, poll=poll, request=request)

    if 'polls' in request.session:
        if poll.pk in request.session['polls']:
            return JsonResponse(vote_data(poll))
    else:
        request.session['polls'] = []

    if form.is_valid():
        form.save()
        request.session['polls'].append(poll.pk)
        request.session.modified = True
        return JsonResponse(vote_data(poll))

    else:
        data = vote_data(poll)
        data.update({'form_error': form.errors})
        return JsonResponse(data)


if getattr(settings, 'WAGTAILPOLLS_VOTE_REQUIRE_PERMS', None):
    vote = permission_required(settings.POLLS_VOTE_REQUIRE_PERMS)(_vote)
else:
    vote = _vote



import datetime

from django import forms
from django.conf import settings
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy as __
from ipware.ip import get_ip, get_real_ip

from .models import Vote


# stop votes from an ip for 1 min after each vote


class SearchForm(forms.Form):
    query = forms.CharField(required=False)


class VoteForm(forms.ModelForm):
    question = forms.ModelChoiceField(Vote.objects.none(), widget=forms.RadioSelect, empty_label=None)

    class Meta:
        model = Vote
        fields = ['question']

    def __init__(self, poll, request=None, data=None, **kwargs):
        super(VoteForm, self).__init__(data, **kwargs)
        self.poll = poll
        self.request = request
        self.fields['question'].queryset = poll.questions.all()
        if request:
            self.ip = get_real_ip(self.request) or get_ip(self.request)

    def recent_vote(self):
        vote = Vote.objects.filter(ip=self.ip).order_by('-time')
        return vote.first()

    def clean(self):
        recent_vote = self.recent_vote()
        cooldown = getattr(settings, 'WAGTAILPOLLS_VOTE_COOLDOWN', 10)
        if not self.ip:
            self.add_error(None, _('Sorry, we were not able to obtain your ip address'))
        if recent_vote is not None:
            if timezone.now() - recent_vote.time < datetime.timedelta(minutes=cooldown):
                error_string = __(
                    'Sorry, you can not vote twice in a minute',
                    'Sorry, you can not vote twice in %(cooldown)s minutes',
                    cooldown) % {
                    'cooldown': cooldown
                }
                self.add_error(None, error_string)

    def save(self, commit=True):
        instance = super(VoteForm, self).save(commit=False)
        instance.ip = self.ip  # TODO
        if commit:
            instance.save()
        return instance



from django.contrib.auth.decorators import permission_required
from django.shortcuts import render, get_object_or_404


@permission_required('wagtailadmin.access_admin')
def results(request, poll_pk):
    from ..models import Poll, Vote

    poll = get_object_or_404(Poll, pk=poll_pk)
    votes = Vote.objects.filter(question__poll=poll)
    total_votes = votes.count()

    return render(request, 'wagtailpolls/poll_results.html', {
        'poll': poll,
        'total_votes': total_votes,
    })



from django import template
from django.core.urlresolvers import reverse

register = template.Library()


@register.simple_tag(takes_context=True)
def querystring(context, **kwargs):
    get = context['request'].GET.copy()
    for key, val in kwargs.items():
        if val is None:
            get.pop(key, None)
        else:
            get[key] = val

    return get.urlencode()


@register.simple_tag(takes_context=True)
def vote(context, poll):
    if poll is None:
        return reverse('wagtailpolls_vote', kwargs={'poll_pk': None})
    return reverse('wagtailpolls_vote', kwargs={'poll_pk': poll.pk})
